package core.notevault;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.navigation.fragment.NavHostFragment;
import com.google.android.material.snackbar.Snackbar;
import com.google.android.material.navigation.NavigationView;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;
import core.notevault.data.Concert;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.NoteSheet;
import core.notevault.databinding.ActivityMainBinding;
import core.notevault.ui.gallery.GalleryFragment;
import core.notevault.ui.gallery.editor.ConcertEditorDialog;
import core.notevault.ui.metadatadialog.MetaDataDialog;
import core.notevault.util.NoteSheetsUtil;

import java.io.*;
import java.nio.file.Files;
import java.util.Date;

public class MainActivity extends AppCompatActivity implements MetaDataDialog.OnMetadataListener, ConcertEditorDialog.OnConcertEditorListener {

    private AppBarConfiguration mAppBarConfiguration;
    private ActivityMainBinding binding;

    private MusicDatabase musicDB;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setSupportActionBar(binding.appBarMain.toolbar);
        DrawerLayout drawer = binding.drawerLayout;
        NavigationView navigationView = binding.navView;
        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_home, R.id.nav_gallery, R.id.nav_slideshow)
                .setOpenableLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        NavigationUI.setupActionBarWithNavController(this, navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);

        musicDB = MusicDatabase.getDatabase(this);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onSupportNavigateUp() {
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_main);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }

    @Override
    public void onMetadataEntered(Uri[] uris, String title, String composer, int year, String genre) {
        new Thread(() -> {
            MusicNote musicNote = new MusicNote(title, composer, year, genre);
            long musicNoteID = musicDB.musicNoteDao().insert(musicNote);

            for(Uri uri: uris) {
                String filePath = saveImageInternally(uri);
                NoteSheet noteSheet = new NoteSheet(musicNoteID, filePath);
                musicDB.musicNoteDao().insertNoteSheet(noteSheet);
            }
        }).start();
    }

    // Speichere eine Kopie des Bilds im internen Speicher
    private String saveImageInternally(Uri uri) {
        try (InputStream inputStream = getContentResolver().openInputStream(uri)) {
            File imageFile = new File(getFilesDir(), "saved_image_" + System.currentTimeMillis() + ".jpg");
            try (OutputStream outputStream = Files.newOutputStream(imageFile.toPath())) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            return imageFile.getAbsolutePath(); // Pfad speichern, um später darauf zuzugreifen
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public void onConcertEditorClosed(String concertTitle, String concertDate) {
        new Thread(() -> {
            Concert concert = new Concert(concertTitle, concertDate);
            Log.d("ConcertEditor", "Saved Concert: " + concertTitle + " on " + concertDate);
            musicDB.musicNoteDao().insertConcert(concert);

            runOnUiThread(() -> {
                NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment_content_main);
                Fragment currentFragment = navHostFragment != null ? navHostFragment.getChildFragmentManager().getFragments().get(0) : null;

                Log.d("MainActivity", "Test for GalleryFragment");
                Log.d("MainActivity", "Current Fragment: " + currentFragment.getClass().getSimpleName());
                if(currentFragment instanceof GalleryFragment) {
                    Log.d("MainActivity", "GalleryFragment Found");
                    GalleryFragment galleryFragment = (GalleryFragment) currentFragment;
                    galleryFragment.addConcert(concert);
                } else {
                    Log.d("MainActivity", "GalleryFragment Not Found");
                }
            });
        }).start();
    }
}