package core.notevault.ui.home;

import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.OpenableColumns;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import core.notevault.R;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.databinding.FragmentHomeBinding;
import core.notevault.ui.metadatadialog.MetaDataDialog;
import core.notevault.util.NoteSheetsUtil;

import java.util.ArrayList;
import java.util.List;

public class HomeFragment extends Fragment {

    private NoteSongAdapter noteSongAdapter;

    private static final int PICK_FILE_REQUEST_CODE = 1;
    private FragmentHomeBinding binding;
    private HomeViewModel homeViewModel;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
         homeViewModel =
                new ViewModelProvider(this).get(HomeViewModel.class);

        binding = FragmentHomeBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        FloatingActionButton importBtn = root.findViewById(R.id.importMusicNotesBtn);
        importBtn.setOnClickListener(v -> openFileChooser());

        RecyclerView recyclerView = root.findViewById(R.id.note_recycler_view);
        noteSongAdapter = new NoteSongAdapter();
        recyclerView.setAdapter(noteSongAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        new LoadSongTitlesTask().execute();

        return root;
    }

    private class LoadSongTitlesTask extends AsyncTask<Void, Void, List<MusicNote>> {

        @Override
        protected List<MusicNote> doInBackground(Void... voids) {
            MusicDatabase db = MusicDatabase.getDatabase(getContext());
            MusicNoteDAO musicNoteDAO = db.musicNoteDao();
            return musicNoteDAO.getAllNotes();
        }

        @Override
        protected void onPostExecute(List<MusicNote> songs) {
            noteSongAdapter.updateSongTitles(songs); // Aktualisiere den Adapter mit den Titeln
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == PICK_FILE_REQUEST_CODE && resultCode == getActivity().RESULT_OK && data != null) {
            if(data.getClipData() != null) {
                int count = data.getClipData().getItemCount();
                Uri[] uris = new Uri[count];
                for(int i = 0; i < count; i++) {
                    uris[i] =  data.getClipData().getItemAt(i).getUri();
                }
                handleFile(uris);
            } else if(data.getData() != null) {
                Uri uri = data.getData();
                handleFile(uri);
            }
        }
    }

    private void handleFile(Uri... uris) {
        // Hier kannst du die Logik zum Speichern oder Anzeigen der Datei implementieren
        NoteSheetsUtil.sortNoteSheetsByTimestamp(getContext(), uris);

        MetaDataDialog metaDataDialog = new MetaDataDialog();
        metaDataDialog.setFileUri(uris);
        metaDataDialog.show(getParentFragmentManager(), MetaDataDialog.TAG);

        Toast.makeText(getActivity(), "Datei ausgewählt: " + uris[0].getPath(), Toast.LENGTH_SHORT).show();
    }

    private void openFileChooser() {
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.setType("*/*"); // Alle Dateitypen

        String[] mimeTypes = {"application/pdf", "image/png", "image/jpeg"};
        intent.putExtra(Intent.EXTRA_MIME_TYPES, mimeTypes);
        intent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);

        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, PICK_FILE_REQUEST_CODE);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }


}