package core.notevault.ui.home;

import android.content.Intent;
import android.os.AsyncTask;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import core.notevault.R;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.data.NoteSheet;

import java.util.ArrayList;
import java.util.List;

public class NoteSongAdapter extends RecyclerView.Adapter<NoteSongAdapter.NoteViewHolder> {

    private final List<MusicNote> noteTitles;

    public NoteSongAdapter(List<MusicNote> noteTitles) {
        this.noteTitles = noteTitles;
    }

    public NoteSongAdapter() {
        this.noteTitles = new ArrayList<>();
    }

    @NonNull
    @Override
    public NoteViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_song, parent, false);
        return new NoteViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull NoteViewHolder holder, int position) {
        holder.titleTextView.setText(noteTitles.get(position).getTitle());
    }

    @Override
    public int getItemCount() {
        return noteTitles.size();
    }

    public void updateSongTitles(List<MusicNote> songs) {
        this.noteTitles.clear();
        this.noteTitles.addAll(songs);
        notifyDataSetChanged();
    }

    public class NoteViewHolder extends RecyclerView.ViewHolder{
        TextView titleTextView;

        NoteViewHolder(View itemView) {
            super(itemView);
            titleTextView = itemView.findViewById(R.id.note_title);

            titleTextView.setOnClickListener(v -> {
                int position = getAdapterPosition();
                if(position != RecyclerView.NO_POSITION) {
                    MusicNote musicNote = noteTitles.get(position);

                    Intent intent = new Intent(itemView.getContext(), FullScreenImageActivity.class);
                    new LoadNoteSheetsTask(musicNote.getMusicNoteId(),
                            MusicDatabase.getDatabase(itemView.getContext()).musicNoteDao(), new OnNoteSheedsLoadedListener() {
                        @Override
                        public void onNoteSheetsLoaded(List<NoteSheet> noteSheets) {
                            String[] noteSheetFiles = new String[noteSheets.size()];
                            for (int i = 0; i < noteSheets.size(); i++) {
                                noteSheetFiles[i] = noteSheets.get(i).getFilePath();
                            }
                            intent.putExtra("imageUris", noteSheetFiles);
                            itemView.getContext().startActivity(intent);
                        }
                    }).execute();

                }
            });
        }

        public void bind(MusicNote note) {
            titleTextView.setText(note.getTitle());
        }


    }

    public interface OnNoteSheedsLoadedListener {
        void onNoteSheetsLoaded(List<NoteSheet> noteSheets);
    }

    private class LoadNoteSheetsTask extends AsyncTask<Void, Void, List<NoteSheet>> {
        private final long musicNoteId;
        private final MusicNoteDAO musicNoteDAO;
        private final OnNoteSheedsLoadedListener listener;



        public LoadNoteSheetsTask(long musicNoteId, MusicNoteDAO musicNoteDAO, OnNoteSheedsLoadedListener listener) {
            this.musicNoteId = musicNoteId;
            this.musicNoteDAO = musicNoteDAO;
            this.listener = listener;
        }

        @Override
        protected List<NoteSheet> doInBackground(Void... voids) {
            List<NoteSheet> sheets =  musicNoteDAO.getNoteSheetsForMusicSong(this.musicNoteId);
            Log.d("LoadNoteSheetsTask", "Loaded NoteSheets: " + sheets.get(0));
            return sheets;
        }

        @Override
        protected void onPostExecute(List<NoteSheet> sheets) {
            listener.onNoteSheetsLoaded(sheets);
        }
    }
}
