package core.notevault.util;

import android.content.Context;
import android.database.Cursor;
import android.media.ExifInterface;
import android.net.Uri;
import android.provider.MediaStore;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class NoteSheetsUtil {

    private static long getImageTimestamp(Context context, Uri imageUri) {
        try  {
            InputStream inputStream = context.getContentResolver().openInputStream(imageUri);
            ExifInterface exif = new ExifInterface(inputStream);

            String dateTime = exif.getAttribute(ExifInterface.TAG_DATETIME);
            if(dateTime != null) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy:MM:dd HH:mm:ss", Locale.getDefault());
                Date date = format.parse(dateTime);
                return date != null ? date.getTime() : 0;
            } else {
                File file = new File(imageUri.getPath());
                return file.lastModified();
            }
        } catch (IOException | ParseException e) {
            throw new RuntimeException(e);
        }
    }

    public static void sortNoteSheetsByTimestamp(Context context, Uri[] uris) {
        ArrayList<UriTimestamp> uriTimestamps = new ArrayList<>();

        for(Uri uri : uris) {
            long timestamp = getImageTimestamp(context, uri);
            uriTimestamps.add(new UriTimestamp(uri, timestamp));
        }

        Collections.sort(uriTimestamps, Comparator.comparingLong(UriTimestamp::getTimestamp));

        for(int i=0; i<uriTimestamps.size(); i++) {
            uris[i] = uriTimestamps.get(i).getUri();
        }
    }


}
